#!/usr/bin/python3
#
# Copyright (C) 2019 Richard Hughes <richard@hughsie.com>
#
# SPDX-License-Identifier: LGPL-2.1+

import sys
import uuid
import argparse
import struct

CAPSULE_FLAGS_PERSIST_ACROSS_RESET = 0x00010000
CAPSULE_FLAGS_POPULATE_SYSTEM_TABLE = 0x00020000
CAPSULE_FLAGS_INITIATE_RESET = 0x00040000

def main(args):

    # parse GUID from command line
    try:
        guid = uuid.UUID(args.guid)
    except ValueError as e:
        print(e)
        return 1
    try:
        with open(args.bin, 'rb') as f:
            bin_data = f.read()
    except FileNotFoundError as e:
        print(e)
        return 1

    # check if already has header
    hdrsz = struct.calcsize('<16sIII')
    if len(bin_data) >= hdrsz:
        hdr = struct.unpack('<16sIII', bin_data[:hdrsz])
        imgsz = hdr[3]
        if imgsz == len(bin_data):
            print('Replacing existing CAPSULE_HEADER of:')
            guid_mixed = uuid.UUID(bytes_le=hdr[0])
            hdrsz_old = hdr[1]
            flags = hdr[2]
            print('GUID:      %s' % guid_mixed)
            print('HdrSz:     0x%04x' % hdrsz_old)
            print('Flags:     0x%04x' % flags)
            print('PayloadSz: 0x%04x' % imgsz)
            bin_data = bin_data[hdrsz_old:]

    # set header flags
    flags = CAPSULE_FLAGS_PERSIST_ACROSS_RESET | CAPSULE_FLAGS_INITIATE_RESET
    if args.flags:
        flags = int(args.flags, 16)

    # build update capsule header
    imgsz = hdrsz + len(bin_data)
    hdr = struct.pack('<16sIII', guid.bytes_le, hdrsz, flags, imgsz)
    with open(args.cap, 'wb') as f:
        f.write(hdr + bin_data)
    print('Wrote capsule %s' % args.cap)
    print('GUID:      %s' % guid)
    print('HdrSz:     0x%04x' % hdrsz)
    print('Flags:     0x%04x' % flags)
    print('PayloadSz: 0x%04x' % imgsz)
    return 0

parser = argparse.ArgumentParser(description='Add capsule header on firmware')
parser.add_argument('--guid', help='GUID of the device', required=True)
parser.add_argument('--bin', help='Path to the .bin file', required=True)
parser.add_argument('--cap', help='Output capsule file path', required=True)
parser.add_argument('--flags', help='Flags, e.g. 0x40000', default=None)
args = parser.parse_args()

sys.exit(main(args))



#!/usr/bin/python3
# pylint: disable=wrong-import-position,too-many-locals,unused-argument,wrong-import-order
#
# Copyright (C) 2017 Richard Hughes <richard@hughsie.com>
#
# SPDX-License-Identifier: LGPL-2.1+

import gi
import os
import requests
import time

gi.require_version('Fwupd', '2.0')

from gi.repository import Fwupd
from gi.repository import Gio
from gi.repository import GLib

def _get_by_device_guid(client, guid):
    cancellable = Gio.Cancellable.new()
    devices = client.get_devices(cancellable)
    for d in devices:
        if d.has_guid(guid):
            return d
    return None

def _get_cache_file(fn):
    cachedir = os.path.expanduser('~/.cache/fwupdmgr')
    if not os.path.exists(cachedir):
        os.makedirs(cachedir)
    cachefn = os.path.join(cachedir, fn)
    if not os.path.exists(cachefn):
        url = 'https://fwupd.org/downloads/' +  fn
        print("Downloading", url)
        r = requests.get(url)
        r.raise_for_status()
        f = open(cachefn, 'wb')
        f.write(r.content)
        f.close()
    return cachefn

class Test:
    def __init__(self, name, guid, has_runtime=True):
        self.files = []
        self.name = name
        self.guid = guid
        self.has_runtime = has_runtime

    def run(self):

        # connect to fwupd
        client = Fwupd.Client.new()
        dev = _get_by_device_guid(client, self.guid)
        if not dev:
            print("Skipping hardware test, no", self.name, "attached")
            return

        print(dev.get_name(), "is currently version", dev.get_version())

        # apply each file
        for fn, ver in self.files:
            fn_cache = _get_cache_file(fn)
            if dev.get_version() == ver:
                flags = Fwupd.InstallFlags.ALLOW_REINSTALL
            else:
                flags = Fwupd.InstallFlags.ALLOW_OLDER
            cancellable = Gio.Cancellable.new()
            print("Installing", fn_cache)
            client.install(dev.get_id(), fn_cache, flags, cancellable)

            # verify version
            if self.has_runtime:
                dev = _get_by_device_guid(client, self.guid)
                if not dev:
                    raise GLib.Error('Device did not come back: ' + self.name)
                if not dev.get_version():
                    raise GLib.Error('No version set after flash for: ' + self.name)
                if dev.get_version() != ver:
                    raise GLib.Error('Got: ' + dev.get_version() + ', expected: ' + ver)

            # FIXME: wait for device to settle?
            time.sleep(2)

    def add_file(self, fn, ver):
        self.files.append((fn, ver))

if __name__ == '__main__':

    tests = []

    # DFU A3BU XPLAINED Mouse
    test = Test('DfuXmegaA3BU-Xplained', '80478b9a-3643-5e47-ab0f-ed28abe1019d')
    test.add_file('f5bbeaba1037dce31dd12f349e8148ae35f98b61-a3bu-xplained123.cab', '1.23')
    test.add_file('24d838541efe0340bf67e1cc5a9b95526e4d3702-a3bu-xplained124.cab', '1.24')
    tests.append(test)

    # DFU AT90USBKEY Mouse
    test = Test('DfuAT90USBKEY', 'c1874c52-5f6a-5864-926d-ea84bcdc82ea')
    test.add_file('b6bef375597e848971f230cf992c9740f7bf5b92-at90usbkey123.cab', '1.23')
    test.add_file('47807fd4a94a4d5514ac6bf7a73038e00ed63225-at90usbkey124.cab', '1.24')
    tests.append(test)

    # Logitech K780 Keyboard
    test = Test('LogitechMPK01', '3932ba15-2bbe-5bbb-817e-6c74e7088509')
    test.add_file('d81a81e13952e871ca2eb86cba7e66199e576a38-Logitech-K780-MPK01.02_B0021.cab', 'MPK01.02_B0021')
    test.add_file('b0dffe84c6d3681e7ae5f27509781bc1cf924dd7-Logitech-K780-MPK01.03_B0024.cab', 'MPK01.03_B0024')
    tests.append(test)

    # Hughski ColorHug (a special variant) using 'dfu'
    test = Test('ColorHugDFU', 'dfbaaded-754b-5214-a5f2-46aa3331e8ce')
    test.add_file('77b315dcaa7edc1d5fbb77016b94d8a0c0133838-fakedevice01_dfu.cab', '0.1')
    test.add_file('8bc3afd07a0af3baaab8b19893791dd3972e8305-fakedevice02_dfu.cab', '0.2')
    tests.append(test)

    # Hughski ColorHug2 using 'colorhug'
    test = Test('ColorHug2', '2082b5e0-7a64-478a-b1b2-e3404fab6dad')
    test.add_file('170f2c19f17b7819644d3fcc7617621cc3350a04-hughski-colorhug2-2.0.6.cab', '2.0.6')
    test.add_file('0a29848de74d26348bc5a6e24fc9f03778eddf0e-hughski-colorhug2-2.0.7.cab', '2.0.7')
    tests.append(test)

    # Logitech Unifying Receiver (RQR12) using 'unifying'
    test = Test('UnifyingRQR12', '9d131a0c-a606-580f-8eda-80587250b8d6')
    test.add_file('6e5ab5961ec4c577bff198ebb465106e979cf686-Logitech-Unifying-RQR12.05_B0028.cab', 'RQR12.05_B0028')
    test.add_file('938fec082652c603a1cdafde7cd25d76baadc70d-Logitech-Unifying-RQR12.07_B0029.cab', 'RQR12.07_B0029')
    tests.append(test)

    # Logitech Unifying Receiver (RQR24) using 'unifying'
    test = Test('UnifyingRQR24', 'cc4cbfa9-bf9d-540b-b92b-172ce31013c1')
    test.add_file('82b90b2614a9a4d0aced1ab8a4a99e228c95585c-Logitech-Unifying-RQ024.03_B0027.cab', 'RQR24.03_B0027')
    test.add_file('4511b9b0d123bdbe8a2007233318ab215a59dfe6-Logitech-Unifying-RQR24.05_B0029.cab', 'RQR24.05_B0029')
    tests.append(test)

    # Jabra Speak 510
    test = Test('JabraSpeak510', '443b9b32-7603-5c3a-bb30-291a7d8d6dbd')
    test.add_file('45f88c50e79cfd30b6599df463463578d52f2fe9-Jabra-SPEAK_510-2.10.cab', '2.10')
    test.add_file('c0523a98ef72508b5c7ddd687418b915ad5f4eb9-Jabra-SPEAK_510-2.14.cab', '2.14')
    tests.append(test)

    # Jabra Speak 410
    test = Test('JabraSpeak410', '1764c519-4723-5514-baf9-3b42970de487')
    test.add_file('eab97d7e745e372e435dbd76404c3929730ac082-Jabra-SPEAK_410-1.8.cab', '1.8')
    test.add_file('50a03efc5df333a948e159854ea40e1a3786c34c-Jabra-SPEAK_410-1.11.cab', '1.11')
    tests.append(test)

    # Jabra Speak 710
    test = Test('JabraSpeak710', '0c503ad9-4969-5668-81e5-a3748682fc16')
    test.add_file('d2910cdbc45cf172767d05e60d9e39a07a10d242-Jabra-SPEAK_710-1.10.cab', '1.10')
    test.add_file('a5c627ae42de4e5c3ae3df28977f480624f96f66-Jabra-SPEAK_710-1.28.cab', '1.28')
    tests.append(test)

    # 8Bitdo SFC30 Gamepad
    test = Test('8BitdoSFC30', 'a7fcfbaf-e9e8-59f4-920d-7691dc6c8699')
    test.add_file('fe066b57c69265f4cce8a999a5f8ab90d1c13b24-8Bitdo-SFC30_NES30_SFC30_SNES30-4.01.cab', '4.01')
    tests.append(test)

    # 8Bitdo NES30Pro Gamepad
    test = Test('8BitdoNES30Pro', 'c6566b1b-0c6e-5d2e-9376-78c23ab57bf2')
    test.add_file('1cb9a0277f536ecd81ca1cea6fd80d60cdbbdcd8-8Bitdo-SFC30PRO_NES30PRO-4.01.cab', '4.01')
    tests.append(test)

    # 8Bitdo SF30 Pro Gamepad
    test = Test('8BitdoSF30Pro', '269b3121-097b-50d8-b9ba-d1f64f9cd241')
    test.add_file('3d3a65ee2e8581647fb09d752fa7e21ee1566481-8Bitdo-SF30_Pro-SN30_Pro-1.26.cab', '1.26')
    tests.append(test)

    # AIAIAI H05
    test = Test('AIAIAI-H05', '7e8318e1-27ae-55e4-a7a7-a35eff60e9bf', has_runtime=False)
    test.add_file('84279d6bab52262080531acac701523604f3e649-AIAIAI-H05-1.6.cab', '1.6')
    tests.append(test)

    # run each test
    rc = 0
    for test in tests:
        try:
            test.run()
        except GLib.Error as e:
            print(str(e))
            rc = 1
        except requests.exceptions.HTTPError as e:
            print(str(e))
            rc = 1



#!/usr/bin/python3
#
# Copyright (C) 2017 Dell, Inc.
#
# SPDX-License-Identifier: LGPL-2.1+
#
from base64 import b64decode
import io
import os
import subprocess
import sys
import shutil
import tempfile
import zipfile
TAG = b'#\x00'

def parse_args():
    import argparse
    parser = argparse.ArgumentParser(description="Self extracting firmware updater")
    parser.add_argument("--directory", help="Directory to extract to")
    parser.add_argument("--cleanup", action='store_true', help="Remove tools when done with installation")
    parser.add_argument("--verbose", action='store_true', help="Run the tool in verbose mode")
    parser.add_argument("--allow-reinstall", action='store_true', help="Allow re-installing existing firmware versions")
    parser.add_argument("--allow-older", action='store_true', help="Allow downgrading firmware versions")
    parser.add_argument("command", choices=["install", "extract"], help="Command to run")
    args = parser.parse_args()
    return args

def error (msg):
    print(msg)
    sys.exit(1)

def bytes_slicer(length, source):
    start = 0
    stop = length
    while start < len(source):
        yield source[start:stop]
        start = stop
        stop += length

def get_zip():
    script = os.path.realpath (__file__)
    bytes_out = io.BytesIO()
    with open(script, 'rb') as source:
        for line in source:
            if not line.startswith(TAG):
                continue
            bytes_out.write(b64decode(line[len(TAG):-1]))
    return bytes_out

def unzip (destination):
    zipf = get_zip ()
    source = zipfile.ZipFile (zipf, 'r')
    for item in source.namelist():
        # extract handles the sanitization
        source.extract (item, destination)

def copy_cabs (source, target):
    if not os.path.exists (target):
        os.makedirs (target)
    cabs = []
    for root, dirs, files in os.walk (source):
        for f in files:
            if (f.endswith ('.cab')):
                origf = os.path.join(root, f)
                shutil.copy (origf, target)
                cabs.append (os.path.join (target, f))
    return cabs


def install_snap (directory, verbose, allow_reinstall, allow_older, uninstall):
    app = 'fwupd'
    common = '/root/snap/%s/common' % app

    #check if snap is installed
    with open(os.devnull, 'w') as devnull:
        subprocess.run (['snap'], check=True, stdout=devnull, stderr=devnull)

    #check existing installed
    cmd = ['snap', 'list', app]
    with open(os.devnull, 'w') as devnull:
        if verbose:
            print(cmd)
        ret = subprocess.run (cmd, stdout=devnull, stderr=devnull)
        if ret.returncode == 0:
            cmd = ['snap', 'remove', app]
            if verbose:
                print(cmd)
            subprocess.run (cmd, check=True)

    # install the snap
    cmd = ['snap', 'ack', os.path.join (directory, 'fwupd.assert')]
    if verbose:
        print(cmd)
    subprocess.run (cmd, check=True)
    cmd = ['snap', 'install', '--classic', os.path.join (directory, 'fwupd.snap')]
    if verbose:
        print(cmd)
    subprocess.run (cmd, check=True)

    # copy the CAB files
    cabs = copy_cabs (directory, common)

    # run the snap
    for cab in cabs:
        cmd = ["%s.fwupdmgr" % app, 'install', cab]
        if allow_reinstall:
            cmd += ["--allow-reinstall"]
        if allow_older:
            cmd += ["--allow-older"]
        if verbose:
            cmd += ["--verbose"]
            print(cmd)
        subprocess.run (cmd)

    #remove copied cabs
    for f in cabs:
        os.remove(f)

    #cleanup
    if uninstall:
        cmd = ['snap', 'remove', app]
        if verbose:
            print(cmd)
        subprocess.run (cmd)

def install_flatpak (directory, verbose, allow_reinstall, allow_older, uninstall):
    app = 'org.freedesktop.fwupd'
    common = '%s/.var/app/%s' % (os.getenv ('HOME'), app)

    with open(os.devnull, 'w') as devnull:
        if not verbose:
            output = devnull
        else:
            output = None
        #look for dependencies
        dep = 'org.gnome.Platform/x86_64/3.30'
        repo = 'flathub'
        repo_url = 'https://flathub.org/repo/flathub.flatpakrepo'
        cmd = ['flatpak', 'info', dep]
        if verbose:
            print(cmd)
        ret = subprocess.run (cmd, stdout=output, stderr=output)
        #not installed
        if ret.returncode != 0:
            #look for remotes
            cmd = ['flatpak', 'remote-info', repo, dep]
            if verbose:
                print(cmd)
            ret = subprocess.run (cmd, stdout=output, stderr=output)
            #not enabled, enable it
            if ret.returncode != 0:
                cmd = ['flatpak', 'remote-add', repo, repo_url]
                if verbose:
                    print(cmd)
                ret = subprocess.run (cmd, stderr=output)
            # install dep
            cmd = ['flatpak', 'install', repo, dep]
            if verbose:
                print(cmd)
            ret = subprocess.run (cmd)

        #check existing installed
        cmd = ['flatpak', 'info', app]
        if verbose:
            print(cmd)
        ret = subprocess.run (cmd, stdout=output, stderr=output)
        if ret.returncode == 0:
            cmd = ['flatpak', 'remove', app]
            if verbose:
                print(cmd)
            subprocess.run (cmd, check=True)

    #install the flatpak
    cmd = ['flatpak', 'install', os.path.join (directory, 'fwupd.flatpak')]
    if verbose:
        print(cmd)
    subprocess.run (cmd, check=True)

    # copy the CAB files
    cabs = copy_cabs (directory, common)

    #run command
    for cab in cabs:
        cmd = ['flatpak', 'run', app, 'install', cab]
        if allow_reinstall:
            cmd += ["--allow-reinstall"]
        if allow_older:
            cmd += ["--allow-older"]
        if verbose:
            cmd += ["--verbose"]
            print(cmd)
        subprocess.run (cmd)

    #remove copied cabs
    for f in cabs:
        os.remove(f)

    #cleanup
    if uninstall:
        cmd = ['flatpak', 'remove', app]
        if verbose:
            print(cmd)
        subprocess.run (cmd)

# Check which package to use
# - return False to use packaged version
# - return True for snap/flatpak
def use_included_version(minimum_version):
    try:
        import apt
    except ModuleNotFoundError:
        return True
    cache = apt.Cache()
    pkg = cache.get("fwupd")
    version = pkg.installed
    if not version:
        return True
    if minimum_version:
        if minimum_version > version:
            print("fwupd %s is already installed but this package requires %s" %
                  (version.version, minimum_version))
        else:
            print("Using existing fwupd version %s already installed on system." % version.version)
            return False
    else:
        print("fwupd %s is installed and must be removed" % version.version)
    return remove_packaged_version(pkg, cache)

def remove_packaged_version(pkg, cache):
    res = False
    while True:
        res = input("Remove now (Y/N)? ")
        if res.lower() == 'n':
            res = False
            break
        if res.lower() == 'y':
            res = True
            break
    if res:
        pkg.mark_delete()
        res = cache.commit()
    if not res:
        raise Exception("Need to remove packaged version")
    return True

def install_builtin(directory, verbose, allow_reinstall, allow_older):
    cabs = []
    for root, dirs, files in os.walk (directory):
        for f in files:
            if f.endswith('.cab'):
                cabs.append(os.path.join(root, f))
    #run command
    for cab in cabs:
        cmd = ['fwupdmgr', 'install', cab]
        if allow_reinstall:
            cmd += ["--allow-reinstall"]
        if allow_older:
            cmd += ["--allow-older"]
        if verbose:
            cmd += ["--verbose"]
            print(cmd)
        subprocess.run(cmd)

def run_installation (directory, verbose, allow_reinstall, allow_older, uninstall):
    try_snap = False
    try_flatpak = False

    #determine if a minimum version was specified
    minimum_path = os.path.join(directory, "minimum")
    minimum = None
    if os.path.exists(minimum_path):
        with open(minimum_path, "r") as rfd:
            minimum = rfd.read()

    if not use_included_version(minimum):
        install_builtin(directory, verbose, allow_reinstall, allow_older)
        return

    # determine what self extracting binary has
    if os.path.exists (os.path.join (directory, 'fwupd.snap')) and \
        os.path.exists (os.path.join (directory, 'fwupd.assert')):
        try_snap = True
    if os.path.exists (os.path.join (directory, 'fwupd.flatpak')):
        try_flatpak = True

    if try_snap:
        try:
            install_snap (directory, verbose, allow_reinstall, allow_older, uninstall)
            return True
        except Exception as _:
            if verbose:
                print ("Snap installation failed")
            if not try_flatpak:
                error ("Snap installation failed")
    if try_flatpak:
        install_flatpak (directory, verbose, allow_reinstall, allow_older, uninstall)

if __name__ == '__main__':
    args = parse_args()
    if 'extract' in args.command:
        if args.allow_reinstall:
            error ("allow-reinstall argument doesn't make sense with command %s" % args.command)
        if args.allow_older:
            error ("allow-older argument doesn't make sense with command %s" % args.command)
        if args.cleanup:
            error ("Cleanup argument doesn't make sense with command %s" % args.command)
        if args.directory is None:
            error ("No directory specified")
        if not os.path.exists (args.directory):
            print ("Creating %s" % args.directory)
            os.makedirs (args.directory)
        unzip (args.directory)
    else:
        if args.directory:
            error ("Directory argument %s doesn't make sense with command %s" % (args.directory, args.command))
        if os.getuid() != 0:
            error ("This tool must be run as root")
        with tempfile.TemporaryDirectory (prefix='fwupd') as target:
            unzip (target)
            run_installation (target, args.verbose, args.allow_reinstall, args.allow_older, args.cleanup)



#!/usr/bin/python3
#
# Copyright (C) 2017 Dell, Inc.
#
# SPDX-License-Identifier: LGPL-2.1+
#
import os
import sys
import xml.etree.ElementTree as etree

def parse_control_dependencies(requested_type):
    TARGET=os.getenv('OS')
    deps = []
    dep = ''

    if TARGET == '':
        print("Missing OS environment variable")
        sys.exit(1)
    OS = TARGET
    SUBOS = ''
    if TARGET:
        split = TARGET.split('-')
        if len(split) >= 2:
            OS = split[0]
            SUBOS = split[1]
    else:
        import lsb_release
        OS = lsb_release.get_distro_information()['ID'].lower()
        import platform
        SUBOS = platform.machine()

    tree = etree.parse(os.path.join(os.path.dirname (sys.argv[0]), "dependencies.xml"))
    root = tree.getroot()
    for child in root:
        if not "type" in child.attrib or not "id" in child.attrib:
            continue
        for distro in child:
            if not "id" in distro.attrib:
                continue
            if distro.attrib["id"] != OS:
                continue
            control = distro.find("control")
            if control is None:
                continue
            packages = distro.findall("package")
            for package in packages:
                if SUBOS:
                    if not 'variant' in package.attrib:
                        continue
                    if package.attrib['variant'] != SUBOS:
                        continue
                if package.text:
                    dep = package.text
                else:
                    dep = child.attrib["id"]
                if child.attrib["type"] == requested_type and dep:
                    version = control.find('version')
                    if version is not None:
                        dep = "%s %s" % (dep, version.text)
                    inclusions = control.findall('inclusive')
                    if inclusions:
                        for i in range(0, len(inclusions)):
                            prefix = ''
                            suffix = ' '
                            if i == 0:
                                prefix = " ["
                            if i == len(inclusions) - 1:
                                suffix = "]"
                            dep = "%s%s%s%s" % (dep, prefix, inclusions[i].text, suffix)
                    exclusions = control.findall('exclusive')
                    if exclusions:
                        for i in range(0, len(exclusions)):
                            prefix = '!'
                            suffix = ' '
                            if i == 0:
                                prefix = " [!"
                            if i == len(exclusions) - 1:
                                suffix = "]"
                            dep = "%s%s%s%s" % (dep, prefix, exclusions[i].text, suffix)
                    deps.append(dep)
    return deps

def update_debian_control(target):
    control_in = os.path.join(target, 'control.in')
    control_out = os.path.join(target, 'control')

    if not os.path.exists(control_in):
        print("Missing file %s" % control_in)
        sys.exit(1)

    with open(control_in, 'r') as rfd:
        lines = rfd.readlines()

    deps = parse_control_dependencies("build")
    deps.sort()
    with open(control_out, 'w') as wfd:
        for line in lines:
            if line.startswith("Build-Depends: %%%DYNAMIC%%%"):
                wfd.write("Build-Depends:\n")
                for i in range(0, len(deps)):
                    wfd.write("\t%s,\n" % deps[i])
            else:
                wfd.write(line)

def update_debian_copyright (directory):
    copyright_in = os.path.join(directory, 'copyright.in')
    copyright_out = os.path.join(directory, 'copyright')

    if not os.path.exists(copyright_in):
        print("Missing file %s" % copyright_in)
        sys.exit(1)

    # Assume all files are remaining LGPL-2.1+
    copyrights = []
    for root, dirs, files in os.walk('.'):
        for file in files:
            target = os.path.join (root, file)
            #skip translations and license file
            if target.startswith('./po/') or file == "COPYING":
                continue
            try:
                with open(target, 'r') as rfd:
                    #read about the first few lines of the file only
                    lines = rfd.readlines(220)
            except UnicodeDecodeError:
                continue
            for line in lines:
                if 'Copyright (C) ' in line:
                    parts = line.split ('Copyright (C)')[1].strip() #split out the copyright header
                    partition = parts.partition(' ')[2] # remove the year string
                    copyrights += ["%s" % partition]
    copyrights = "\n\t   ".join(sorted(set(copyrights)))
    with open(copyright_in, 'r') as rfd:
        lines = rfd.readlines()

    with open(copyright_out, 'w') as wfd:
        for line in lines:
            if line.startswith("%%%DYNAMIC%%%"):
                wfd.write("Files: *\n")
                wfd.write("Copyright: %s\n" % copyrights)
                wfd.write("License: LGPL-2.1+\n")
                wfd.write("\n")
            else:
                wfd.write(line)

directory = os.path.join (os.getcwd(), 'debian')
update_debian_control(directory)
update_debian_copyright(directory)



#!/usr/bin/python3
#
# Copyright (C) 2017 Dell, Inc.
#
# SPDX-License-Identifier: LGPL-2.1+
#
from base64 import b64encode
import io
import os
import subprocess
import shutil
import sys
import tempfile
import zipfile
from assets.header import TAG

def error (msg):
    print(msg)
    sys.exit(1)

def parse_args():
    import argparse
    parser = argparse.ArgumentParser(description="Generate a standalone firmware updater")
    parser.add_argument("--disable-snap-download", action='store_true', help="Don't download support for snap")
    parser.add_argument("--disable-flatpak-download", action='store_true', help="Don't download support for flatpak")
    parser.add_argument("--snap-channel", help="Channel to download snap from (optional)")
    parser.add_argument("--minimum", help="Use already installed fwupd version if at least this version")
    parser.add_argument("cab", help="CAB file or directory containing CAB files to automatically install")
    parser.add_argument('target', help='target file to create')
    args = parser.parse_args()
    return args

def bytes_slicer(length, source):
    start = 0
    stop = length
    while start < len(source):
        yield source[start:stop]
        start = stop
        stop += length

def generate_installer (directory, target):
    asset_base = os.path.join (os.path.dirname(os.path.realpath(__file__)),
                               "assets")

    #header
    shutil.copy (os.path.join (asset_base, "header.py"), target)

    #zip file
    buffer = io.BytesIO()
    archive = zipfile.ZipFile(buffer, "a")
    for root, dirs, files in os.walk (directory):
        for f in files:
            source = os.path.join(root, f)
            archive_fname = source.split (directory) [1]
            archive.write(source, archive_fname)
    if 'DEBUG' in os.environ:
        print (archive.namelist())
    archive.close()

    with open (target, 'ab') as bytes_out:
        encoded = b64encode(buffer.getvalue())
        for section in bytes_slicer(64, encoded):
            bytes_out.write(TAG)
            bytes_out.write(section)
            bytes_out.write(b'\n')

def download_snap (directory, channel):
    cmd = ['snap', 'download', 'fwupd']
    if channel is not None:
        cmd += ['--channel', channel]
    if 'DEBUG' in os.environ:
        print(cmd)
    subprocess.run (cmd, cwd=directory, check=True)
    for f in os.listdir (directory):
        # the signatures associated with the snap
        if f.endswith(".assert"):
            shutil.move (os.path.join(directory, f), os.path.join(directory, 'fwupd.assert'))
        # the snap binary itself
        elif f.endswith(".snap"):
            shutil.move (os.path.join(directory, f), os.path.join(directory, 'fwupd.snap'))

def download_cab_file (directory, uri):
    cmd = ['wget', uri]
    if 'DEBUG' in os.environ:
        print(cmd)
    subprocess.run (cmd, cwd=directory, check=True)

def download_flatpak (directory):
    dep = 'org.freedesktop.fwupd'
    flatpak_dir = os.path.join(os.getenv('HOME'),'.local', 'share', 'flatpak')
    verbose = 'DEBUG' in os.environ

    #check if we have installed locally already or not
    if not os.path.exists (os.path.join (flatpak_dir, 'app', dep)):
        # install into local user's repo
        cmd = ['flatpak', 'install', '--user',
            'https://www.flathub.org/repo/appstream/org.freedesktop.fwupd.flatpakref', '--no-deps', '-y']
        if verbose:
            print(cmd)
        subprocess.run (cmd, cwd=directory, check=True)

    # generate a bundle
    repo = os.path.join(flatpak_dir, 'repo')
    cmd = ['flatpak', 'build-bundle', repo, 'fwupd.flatpak', dep, 'stable']
    if verbose:
        print(cmd)
    subprocess.run (cmd, cwd=directory, check=True)

if __name__ == '__main__':
    args = parse_args()

    if not args.cab.startswith("http"):
        local = args.cab

    with tempfile.TemporaryDirectory (prefix='fwupd') as directory:
        if local:
            if not os.path.exists (local):
                error ("%s doesn't exist" % local)
            if not os.path.isdir(local):
                shutil.copy (local, directory)
            else:
                for root, dirs, files in os.walk(local):
                    for f in files:
                        shutil.copy (os.path.join(root, f), directory)
        else:
            download_cab_file (directory, args.cab)

        if not args.disable_snap_download:
            download_snap (directory, args.snap_channel)

        if not args.disable_flatpak_download:
            download_flatpak (directory)

        if args.minimum:
            with open(os.path.join(directory, "minimum"), "w") as wfd:
                wfd.write(args.minimum)

        generate_installer (directory, args.target)



#!/usr/bin/python3
# SPDX-License-Identifier: LGPL-2.1+

import csv
import binascii
import os
import struct
import glob
from collections import namedtuple

class Record(object):
    def __init__(self, filename, cns):
        self.filename = filename
        self.cns = cns

def load_pci_ids():
    pci_vendors = {}
    pci_vendors[0x1987] = 'Freescale'
    for ln in open('/usr/share/hwdata/pci.ids').read().split('\n'):
        if ln.startswith('#'):
            continue
        if ln.startswith('\t'):
            continue
        data = ln.split('  ')
        if len(data) != 2:
            continue
        pci_vendors[int(data[0], 16)] = data[1].split(' ')[0]
        if data[0] == 'ffff':
            break
    return pci_vendors

def _data_to_utf8(s):
    return s.decode('utf-8', 'replace').replace('\0', ' ')

def main():

    # open files
    records = []
    for fn in glob.glob('tests/nvme/*'):
        blob = open(fn, 'rb').read()
        if len(blob) != 4096:
            print('WARNING: ignoring %s of size %i' % (fn, len(blob)))
            continue
        Cns = namedtuple('Cns',
                         'vid ssvid sn mn fr rab ieee cmic mdts cntlid ver ' \
                         'rtd3r rtd3e oaes ctratt rrls rsvd102 oacs acl aerl ' \
                         'frmw lpa elpe npss avscc apsta wctemp cctemp mtfa ' \
                         'hmpre hmmin tnvmcap unvmcap rpmbs edstt dsto fwug ' \
                         'kas hctma mntmt mxtmt sanicap hmminds hmmaxd ' \
                         'nsetidmax rsvd340 anatt anacap anagrpmax nanagrpid ' \
                         'rsvd352 sqes cqes maxcmd nn oncs fuses fna vwc awun ' \
                         'awupf nvscc nwpc acwu rsvd534 sgls mnan rsvd544 ' \
                         'subnqn rsvd1024 ioccsz iorcsz icdoff ctrattr msdbd ' \
                         'rsvd1804 psd vs')
        try:
            cns = Cns._make(struct.unpack('<HH20s40s8sB3pBBHIIIIIH154pHBBBBBBBBHHHII16p' \
                                          '16pIHBBHHHHIIHH2pBBII160pBBHIHHBBHHBBH2pII' \
                                          '224p256p768pIIHBB244p1024p1024p', blob))
        except struct.error as e:
            print('WARNING: ignoring %s of size %i' % (fn, len(blob)))
            continue
        records.append(Record(fn, cns))

    # try to sort in sane way
    records = sorted(records,
                     key=lambda k: str(k.cns.vid) + k.cns.mn.decode('utf-8', 'replace') + k.cns.sn.decode('utf-8', 'replace'),
                     reverse=True)

    # export csv
    with open('all.csv', 'w', newline='') as csvfile:
        exp = csv.writer(csvfile)
        exp.writerow(['id', 'vid', 'sn', 'mn', 'fr',
                      'rrls', 'frmw', 'fwug', 'subnqn', 'vs'])
        for r in records:
            cns = r.cns
            sn = cns.sn.decode('utf-8', 'replace').replace('\0', ' ')
            mn = cns.mn.decode('utf-8', 'replace').replace('\0', ' ')
            fr = cns.fr.decode('utf-8', 'replace').replace('\0', ' ')
            exp.writerow([os.path.basename(r.filename)[:6],
                          '%04x' % cns.vid,
                          sn, mn, fr, cns.rrls,
                          '%02x' % cns.frmw,
                          cns.fwug, cns.subnqn,
                          binascii.hexlify(cns.vs)])

    # frmw stats
    s1ro_cnt = 0
    fawr_cnt = 0
    nfws_map = {}
    for r in records:
        if r.cns.frmw & 0x01 > 0:
            s1ro_cnt += 1
        if (r.cns.frmw & 0x10) >> 4:
            fawr_cnt += 1
        nfws = (r.cns.frmw & 0x0e) >> 1
        if nfws in nfws_map:
            nfws_map[nfws] += 1
            continue
        nfws_map[nfws] = 1
    print('s1ro=%i/%i' % (s1ro_cnt, len(records)))
    print('fawr=%i/%i' % (fawr_cnt, len(records)))
    nfws = sorted(nfws_map.items(), key=lambda k: k[0], reverse=True)
    for nfws, cnt in nfws:
        print('nfws[%i]=%i' % (nfws, cnt))

    # vendor popularity
    vids = {}
    for r in records:
        if r.cns.vid not in vids:
            vids[r.cns.vid] = 1
            continue
        vids[r.cns.vid] += 1
    vids = sorted(vids.items(), key=lambda k: k[1], reverse=True)
    pci_vendors = load_pci_ids()
    for vid, cnt in vids:
        name = '0x%04x' % vid
        if vid in pci_vendors:
            name = pci_vendors[vid]
        print('%s,%i' % (name, cnt))

    # vendor records
    vs_records = []
    for r in records:
        if r.cns.vs:
            vs_records.append(r)
    print('nr_vs=%i' % len(vs_records))

main()



#!/usr/bin/python3
# SPDX-License-Identifier: LGPL-2.1+
"""A simple fwupd frontend"""
import sys
import os
import gi
from gi.repository import GLib
gi.require_version('Fwupd', '2.0')
from gi.repository import Fwupd #pylint: disable=wrong-import-position

class Progress():
    """Class to track the signal changes of progress events"""
    def __init__(self):
        self.device = None
        self.status = None
        self.percent = 0
        self.erase = 0

    def device_changed(self, new_device):
        """Indicate new device string to track"""
        if self.device != new_device:
            self.device = new_device
            print("\nUpdating %s" % self.device)

    def status_changed(self, percent, status):
        """Indicate new status string or % complete to track"""
        if self.status != status or self.percent != percent:
            for i in range(0, self.erase):
                sys.stdout.write("\b \b")
            self.status = status
            self.percent = percent
            status_str = "["
            for i in range(0, 50):
                if i < percent/2:
                    status_str += '*'
                else:
                    status_str += ' '
            status_str += "] %d%% %s" %(percent, status)
            status_str.erase = len(status_str)
            sys.stdout.write(status_str)
            sys.stdout.flush()
            if 'idle' in status:
                sys.stdout.write("\n")

def parse_args():
    """Parse arguments for this client"""
    import argparse
    parser = argparse.ArgumentParser(description="Interact with fwupd daemon")
    parser.add_argument("--allow-older", action="store_true",
                        help="Install older payloads(default False)")
    parser.add_argument("--allow-reinstall", action="store_true",
                        help="Reinstall payloads(default False)")
    parser.add_argument("command", choices=["get-devices",
                                            "get-details",
                                            "install"], help="What to do")
    parser.add_argument('cab', nargs='?', help='CAB file')
    parser.add_argument('deviceid', nargs='?',
                        help='DeviceID to operate on(optional)')
    args = parser.parse_args()
    return args

def get_devices(client):
    """Use fwupd client to fetch devices"""
    devices = client.get_devices()
    for item in devices:
        print(item.to_string())

def get_details(client, cab):
    """Use fwupd client to fetch details for a CAB file"""
    devices = client.get_details(cab, None)
    for device in devices:
        print(device.to_string())

def status_changed(client, spec, progress): #pylint: disable=unused-argument
    """Signal emitted by fwupd daemon indicating status changed"""
    progress.status_changed(client.get_percentage(),
                            Fwupd.status_to_string(client.get_status()))

def device_changed(client, device, progress): #pylint: disable=unused-argument
    """Signal emitted by fwupd daemon indicating active device changed"""
    progress.device_changed(device.get_name())

def install(client, cab, target, older, reinstall):
    """Use fwupd client to install CAB file to applicable devices"""
    # FWUPD_DEVICE_ID_ANY
    if not target:
        target = '*'
    flags = Fwupd.InstallFlags.NONE
    if older:
        flags |= Fwupd.InstallFlags.ALLOW_OLDER
    if reinstall:
        flags |= Fwupd.InstallFlags.ALLOW_REINSTALL
    progress = Progress()
    parent = super(client.__class__, client)
    parent.connect('device-changed', device_changed, progress)
    parent.connect('notify::percentage', status_changed, progress)
    parent.connect('notify::status', status_changed, progress)
    try:
        client.install(target, cab, flags, None)
    except GLib.Error as glib_err: #pylint: disable=catching-non-exception
        progress.status_changed(0, 'idle')
        print("%s" % glib_err)
        sys.exit(1)

def check_cab(cab):
    """Check that CAB file exists"""
    if not cab:
        print("Need to specify payload")
        sys.exit(1)
    if not os.path.isfile(cab):
        print("%s doesn't exist or isn't a file" % cab)
        sys.exit(1)

if __name__ == '__main__':
    ARGS = parse_args()
    CLIENT = Fwupd.Client()
    CLIENT.connect()

    if ARGS.command == "get-devices":
        get_devices(CLIENT)
    elif ARGS.command == "get-details":
        check_cab(ARGS.cab)
        get_details(CLIENT, ARGS.cab)
    elif ARGS.command == "install":
        check_cab(ARGS.cab)
        install(CLIENT, ARGS.cab, ARGS.deviceid, ARGS.allow_older, ARGS.allow_reinstall)



#!/usr/bin/python3
# SPDX-License-Identifier: LGPL-2.1+

import sys
import os
import subprocess

def _do_msgattrib(fn):
    argv = ['msgattrib',
            '--no-location',
            '--translated',
            '--no-wrap',
            '--sort-output',
            fn,
            '--output-file=' + fn]
    ret = subprocess.run(argv)
    if ret.returncode != 0:
        return

def _do_nukeheader(fn):
    clean_lines = []
    with open(fn) as f:
        lines = f.readlines()
    for line in lines:
        if line.startswith('"POT-Creation-Date:'):
            continue
        if line.startswith('"PO-Revision-Date:'):
            continue
        if line.startswith('"Last-Translator:'):
            continue
        clean_lines.append(line)
    with open(fn, 'w') as f:
        f.writelines(clean_lines)

def _process_file(fn):
    _do_msgattrib(fn)
    _do_nukeheader(fn)

if __name__ == '__main__':
    if len(sys.argv) == 1:
        print('path required')
        sys.exit(1)
    try:
        dirname = sys.argv[1]
        for fn in os.listdir(dirname):
            if fn.endswith('.po'):
                _process_file(os.path.join(dirname, fn))
    except NotADirectoryError as _:
        print('path required')
        sys.exit(2)



#!/usr/bin/python3
#
# Copyright (C) 2017-2018 Dell, Inc.
#
# SPDX-License-Identifier: LGPL-2.1+
#
import os
import subprocess
import sys
import xml.etree.ElementTree as etree
import tempfile

def parse_dependencies(OS, SUBOS, requested_type):
    deps = []
    dep = ''
    tree = etree.parse(os.path.join(directory, "dependencies.xml"))
    root = tree.getroot()
    for child in root:
        if not "type" in child.attrib or not "id" in child.attrib:
            continue
        for distro in child:
            if not "id" in distro.attrib:
                continue
            if distro.attrib["id"] != OS:
                continue
            packages = distro.findall("package")
            for package in packages:
                if SUBOS:
                    if not 'variant' in package.attrib:
                        continue
                    if package.attrib['variant'] != SUBOS:
                        continue
                if package.text:
                    dep = package.text
                else:
                    dep = child.attrib["id"]
                if child.attrib["type"] == requested_type and dep:
                    deps.append(dep)
    return deps

directory = os.path.dirname(sys.argv[0])
TARGET=os.getenv('OS')

if TARGET is None:
    print("Missing OS environment variable")
    sys.exit(1)
OS = TARGET
SUBOS = ''
split = TARGET.split('-')
if len(split) >= 2:
    OS = split[0]
    SUBOS = split[1]

deps = parse_dependencies(OS, SUBOS, "build")

input = os.path.join(directory, "Dockerfile-%s.in" % OS)
if not os.path.exists(input):
    print("Missing input file %s for %s" % (input, OS))
    sys.exit(1)

with open(input, 'r') as rfd:
    lines = rfd.readlines()

with open('Dockerfile', 'w') as wfd:
    for line in lines:
        if line.startswith("FROM %%%ARCH_PREFIX%%%"):
            if (OS == "debian" or OS == "ubuntu") and SUBOS == "i386":
                replace = SUBOS + "/"
            else:
                replace = ''
            wfd.write(line.replace("%%%ARCH_PREFIX%%%", replace))
        elif line == "%%%INSTALL_DEPENDENCIES_COMMAND%%%\n":
            if OS == "fedora" or OS == 'flatpak':
                wfd.write("RUN dnf --enablerepo=updates-testing -y install \\\n")
            elif OS == "centos":
                wfd.write("RUN yum -y install \\\n")
            elif OS == "debian" or OS == "ubuntu":
                wfd.write("RUN apt update -qq && \\\n")
                wfd.write("\tapt install -yq --no-install-recommends\\\n")
            elif OS == "arch":
                wfd.write("RUN pacman -Syu --noconfirm \\\n")
            for i in range(0, len(deps)):
                if i < len(deps)-1:
                    wfd.write("\t%s \\\n" % deps[i])
                else:
                    wfd.write("\t%s \n" % deps[i])
        elif line == "%%%ARCH_SPECIFIC_COMMAND%%%\n":
            if OS == "debian" and SUBOS == "s390x":
                #add sources
                wfd.write('RUN cat /etc/apt/sources.list | sed "s/deb/deb-src/" >> /etc/apt/sources.list\n')
                #add new architecture
                wfd.write('RUN dpkg --add-architecture %s\n' % SUBOS)
        elif line == "%%%OS%%%\n":
            wfd.write("ENV OS %s\n" % TARGET)
        else:
            wfd.write(line)
    wfd.flush()
    args = ["docker", "build", "-t", "fwupd-%s" % TARGET]
    if 'http_proxy' in os.environ:
        args += ['--build-arg=http_proxy=%s' % os.environ['http_proxy']]
    if 'https_proxy' in os.environ:
        args += ['--build-arg=https_proxy=%s' % os.environ['https_proxy']]
    args += ["-f", "./Dockerfile", "."]
    subprocess.check_call(args)



#!/usr/bin/python3
#
# Copyright (C) 2021 Javier Martinez Canillas <javierm@redhat.com>
# Copyright (C) 2021 Richard Hughes <richard@hughsie.com>
#
# SPDX-License-Identifier: LGPL-2.1+
#
# pylint: disable=missing-docstring, invalid-name

import subprocess
import sys
import argparse
import tempfile


def _generate_sbat(args):
    """ append SBAT metadata """
    FWUPD_SUMMARY = "Firmware update daemon"
    FWUPD_URL = "https://github.com/fwupd/fwupd"

    subprocess.run(
        [args.cc, "-x", "c", "-c", "-o", args.outfile, "/dev/null"], check=True
    )

    # not specified
    if not args.sbat_distro_id:
        return

    with tempfile.NamedTemporaryFile() as sfd:

        # spec
        sfd.write(
            "{0},{1},{2},{0},{1},{3}\n".format(
                "sbat",
                args.sbat_version,
                "UEFI shim",
                "https://github.com/rhboot/shim/blob/main/SBAT.md",
            ).encode()
        )

        # fwupd
        sfd.write(
            "{0},{1},{2},{0},{3},{4}\n".format(
                args.project_name,
                args.sbat_generation,
                "Firmware update daemon",
                args.project_version,
                FWUPD_URL,
            ).encode()
        )

        # distro specifics, falling back to the project defaults
        sfd.write(
            "{0}-{1},{2},{3},{4},{5},{6}\n".format(
                args.project_name,
                args.sbat_distro_id,
                args.sbat_distro_generation or args.sbat_generation,
                args.sbat_distro_summary or FWUPD_SUMMARY,
                args.sbat_distro_pkgname,
                args.sbat_distro_version or args.project_version,
                args.sbat_distro_url or FWUPD_URL,
            ).encode()
        )

        # all written
        sfd.seek(0)

        # add a section to the object; use `objdump -s -j .sbat` to verify
        argv = [
            args.objcopy,
            "--add-section",
            ".sbat={}".format(sfd.name),
            "--set-section-flags",
            ".sbat=contents,alloc,load,readonly,data",
            args.outfile,
        ]
        subprocess.run(argv, check=True)


if __name__ == "__main__":

    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--cc",
        default="gcc",
        help="Compiler to use for generating sbat object",
    )
    parser.add_argument(
        "--objcopy",
        default="objcopy",
        help="Binary file to use for objcopy",
    )
    parser.add_argument(
        "--project-name",
        help="SBAT project name",
    )
    parser.add_argument(
        "--project-version",
        help="SBAT project version",
    )
    parser.add_argument(
        "--sbat-version",
        default=1,
        type=int,
        help="SBAT version",
    )
    parser.add_argument(
        "--sbat-generation",
        default=1,
        type=int,
        help="SBAT generation",
    )
    parser.add_argument(
        "--sbat-distro-id",
        default=None,
        help="SBAT distribution ID"
    )
    parser.add_argument(
        "--sbat-distro-generation",
        default=None,
        type=int,
        help="SBAT distribution generation",
    )
    parser.add_argument(
        "--sbat-distro-summary",
        default=None,
        help="SBAT distribution summary",
    )
    parser.add_argument(
        "--sbat-distro-pkgname",
        default=None,
        help="SBAT distribution package name",
    )
    parser.add_argument(
        "--sbat-distro-version",
        default=None,
        help="SBAT distribution version",
    )
    parser.add_argument(
        "--sbat-distro-url",
        default=None,
        help="SBAT distribution URL",
    )
    parser.add_argument(
        "outfile",
        help="Output file",
    )
    _args = parser.parse_args()
    _generate_sbat(_args)

    sys.exit(0)



#!/usr/bin/python3
#
# Copyright (C) 2017 Dell, Inc.
# Copyright (C) 2020 Intel, Inc.
#
# SPDX-License-Identifier: LGPL-2.1+
#
import os
import sys
import argparse
import xml.etree.ElementTree as etree


def parse_dependencies(OS, SUBOS, requested_type):
    deps = []
    dep = ''
    directory = os.path.dirname(sys.argv[0])
    tree = etree.parse(os.path.join(directory, "dependencies.xml"))
    root = tree.getroot()
    for child in root:
        if "type" not in child.attrib or "id" not in child.attrib:
            continue
        for distro in child:
            if "id" not in distro.attrib:
                continue
            if distro.attrib["id"] != OS:
                continue
            packages = distro.findall("package")
            for package in packages:
                if SUBOS:
                    if 'variant' not in package.attrib:
                        continue
                    if package.attrib['variant'] != SUBOS:
                        continue
                if package.text:
                    dep = package.text
                else:
                    dep = child.attrib["id"]
                if child.attrib["type"] == requested_type and dep:
                    deps.append(dep)
    return deps


if __name__ == '__main__':

    try:
        import distro

        target = distro.linux_distribution()[0]
    except ModuleNotFoundError:
        target = None

    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-o",
        "--os",
        default=target,
        choices=["fedora", "centos", "flatpak", "debian", "ubuntu", "arch"],
        help="dependencies for OS",
    )
    args = parser.parse_args()

    target = os.getenv('OS', args.os)
    if target is None:
        print("Missing OS environment variable")
        sys.exit(1)

    _os = target.lower()
    _sub_os = ''
    split = target.split('-')
    if len(split) >= 2:
        _os, _sub_os = split[:2]
    dependencies = parse_dependencies(_os, _sub_os, "build")
    print(*dependencies, sep='\n')



#!/usr/bin/python3
""" This parses avrdude.conf and generates quirks for fwupd """

# pylint: disable=wrong-import-position,pointless-string-statement

"""
SPDX-License-Identifier: LGPL-2.1+
"""

import sys
from difflib import SequenceMatcher

# finds a part using the ID
def _find_part_by_id(parts, part_id):
    for part in parts:
        if 'id' not in part:
            continue
        if part['id'] == part_id:
            return part
    return None

# finds a memory layout for a part, climbing up the tree to the parent if reqd.
def _find_mem_layout(parts, part):
    if 'memory-application' in part:
        memory_flash = part['memory-application']
        if memory_flash:
            return memory_flash

    #look at the parent
    if 'parent' in part:
        parent = _find_part_by_id(parts, part['parent'])
        if parent:
            return _find_mem_layout(parts, parent)
        print('no parent ', part['parent'], 'found for', part['id'])
    return None

# parses the weird syntax of avrdude.conf and makes lots of nested dictionaries
def _parse_parts(fn_source):
    print("reading", fn_source)

    part = None
    memory_id = None
    parts = []

    for line in open(fn_source).readlines():

        # try to clean up crazy syntax
        line = line.replace('\n', '')
        if line.endswith(';'):
            line = line[:-1]

        # ignore blank lines
        line = line.rstrip()
        if not line:
            continue

        # count how many spaces deep this is
        lvl = 0
        for char in line:
            if char != ' ':
                break
            lvl = lvl + 1

        # ignore comments
        line = line.strip()
        if line[0] == '#':
            continue

        # level 0 of hell
        if lvl == 0:
            if line.startswith('part'):
                memory_id = None
                part = {}
                parts.append(part)
                if line.startswith('part parent '):
                    part['parent'] = line[13:].replace('"', '')
            continue

        # level 4 of hell
        if lvl == 4:
            if line.startswith('memory'):
                memory_id = 'memory-' + line[7:].replace('"', '')
                part[memory_id] = {}
                continue
            split = line.split('=')
            if len(split) != 2:
                print('ignoring', line)
                continue
            part[split[0].strip()] = split[1].strip().replace('"', '')
            continue

        # level 8 of hell
        if lvl == 8:
            if memory_id:
                split = line.split('=')
                if len(split) != 2:
                    continue
                memory = part[memory_id]
                memory[split[0].strip()] = split[1].strip()
            continue
    return parts

def _get_longest_substring(s1, s2):
    match = SequenceMatcher(None, s1, s2).find_longest_match(0, len(s1), 0, len(s2))
    return s2[match.b: match.b + match.size]

# writes important data to the quirks file
def _write_quirks(parts, fn_destination):
    outp = []

    results = {}

    for part in parts:

        # ignore meta parts with deprecated names
        if 'desc' not in part:
            continue
        if 'signature' not in part:
            continue

        # find the layout
        mem_part = _find_mem_layout(parts, part)
        if not mem_part:
            print("no memory layout for", part['desc'])
            continue
        if not 'size' in mem_part:
            print("no memory size for", part['desc'])
            continue
        if mem_part['size'].startswith('0x'):
            size = int(mem_part['size'], 16)
        else:
            size = int(mem_part['size'], 10)

        # output the line for the quirk
        chip_id = '0x' + part['signature'].replace('0x', '').replace(' ', '')
        mem_layout = '@Flash/0x0/1*%.0iKg' % int(size / 1024)

        # merge duplicate quirks
        if chip_id in results:
            result = results[chip_id]
            result['desc'] = _get_longest_substring(result['desc'], part['desc'])
        else:
            result = {}
            result['desc'] = part['desc']
            result['size'] = size
            result['mem_layout'] = mem_layout
            results[chip_id] = result

    for chip_id in results:
        result = results[chip_id]
        outp.append('# ' + result['desc'] + '	[USER]		USER=0x%x' % result['size'] + '\n')
        outp.append(chip_id + '=' + result['mem_layout'] + '\n\n')

    # write file
    print("writing", fn_destination)
    open(fn_destination, 'w').writelines(outp)

if __name__ == '__main__':
    if len(sys.argv) != 3:
        print("USAGE: %s avrdude.conf tmp.quirk" % sys.argv[0])
        sys.exit(1)

    all_parts = _parse_parts(sys.argv[1])
    _write_quirks(all_parts, sys.argv[2])



#!/usr/bin/python3
#
# Copyright (C) 2021 Javier Martinez Canillas <javierm@redhat.com>
# Copyright (C) 2021 Richard Hughes <richard@hughsie.com>
#
# SPDX-License-Identifier: LGPL-2.1+
#
# pylint: disable=missing-docstring, invalid-name

import subprocess
import sys
import argparse


def _run_objcopy(args):

    argv = [
        args.objcopy,
        "-j",
        ".text",
        "-j",
        ".sbat",
        "-j",
        ".sdata",
        "-j",
        ".data",
        "-j",
        ".dynamic",
        "-j",
        ".dynsym",
        "-j",
        ".rel*",
        args.infile,
        args.outfile,
    ]

    # aarch64 and arm32 don't have an EFI capable objcopy
    # Use 'binary' instead, and add required symbols manually
    if args.arch in ["aarch64", "arm"]:
        argv.extend(["-O", "binary"])
    else:
        argv.extend(["--target", "efi-app-{}".format(args.arch)])
    try:
        subprocess.run(argv, check=True)
    except FileNotFoundError as e:
        print(str(e))
        sys.exit(1)


def _run_genpeimg(args):

    # this is okay if it does not exist
    argv = [
        "genpeimg",
        "-d",
        "+d",
        "+n",
        "-d",
        "+s",
        args.outfile,
    ]
    try:
        subprocess.run(argv, check=True)
    except FileNotFoundError as _:
        pass


if __name__ == "__main__":

    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--objcopy",
        default="objcopy",
        help="Binary file to use for objcopy",
    )
    parser.add_argument(
        "--arch",
        default="x86_64",
        help="EFI architecture",
    )
    parser.add_argument(
        "infile",
        help="Input file",
    )
    parser.add_argument(
        "outfile",
        help="Output file",
    )
    _args = parser.parse_args()
    _run_objcopy(_args)
    _run_genpeimg(_args)

    sys.exit(0)



#!/usr/bin/python3
import subprocess
import os
import json
import shutil

def prepare (target):
    #clone the flatpak json
    cmd = ['git', 'submodule', 'update', '--remote', 'contrib/flatpak']
    subprocess.run (cmd, check=True)

    #clone the submodules for that
    cmd = ['git', 'submodule', 'update', '--init', '--remote', 'shared-modules/']
    subprocess.run (cmd, cwd='contrib/flatpak', check=True)

    #parse json
    if os.path.isdir ('build'):
        shutil.rmtree ('build')
    data = {}
    with open ('contrib/flatpak/org.freedesktop.fwupd.json', 'r') as rfd:
        data = json.load (rfd, strict=False)
    platform = 'runtime/%s/x86_64/%s' % (data['runtime'], data['runtime-version'])
    sdk = 'runtime/%s/x86_64/%s' % (data['sdk'], data['runtime-version'])
    num_modules = len (data['modules'])

    #update to build from master
    data["branch"] = "master"
    for index in range(0, num_modules):
        module = data['modules'][index]
        if type (module) != dict or not 'name' in module:
            continue
        name = module['name']
        if not 'fwupd' in name:
            continue
        data['modules'][index]['sources'][0].pop ('url')
        data['modules'][index]['sources'][0].pop ('sha256')
        data['modules'][index]['sources'][0]['type'] = 'dir'
        data['modules'][index]['sources'][0]['skip'] = [".git"]
        data['modules'][index]['sources'][0]['path'] = ".."

    #write json
    os.mkdir('build')
    with open (target, 'w') as wfd:
        json.dump(data, wfd, indent=4)
    os.symlink ('../contrib/flatpak/shared-modules','build/shared-modules')

    # install the runtimes (parsed from json!)
    repo = 'flathub'
    repo_url = 'https://dl.flathub.org/repo/flathub.flatpakrepo'
    print ("Installing dependencies")
    cmd = ['flatpak', 'remote-add', '--if-not-exists', repo, repo_url]
    subprocess.run (cmd, check=True)
    cmd = ['flatpak', 'install', '--assumeyes', repo, sdk]
    subprocess.run (cmd, check=True)
    cmd = ['flatpak', 'install', '--assumeyes', repo, platform]
    subprocess.run (cmd, check=True)


def build (target):
    cmd = ['flatpak-builder', '--repo=repo', '--force-clean', '--disable-rofiles-fuse', 'build-dir', target]
    subprocess.run (cmd, check=True)
    cmd = ['flatpak', 'build-bundle', 'repo', 'fwupd.flatpak', 'org.freedesktop.fwupd']
    subprocess.run (cmd, check=True)

if __name__ == '__main__':
    t = os.path.join ('build', 'org.freedesktop.fwupd.json')
    prepare (t)
    build (t)

# to run from the builddir:
# sudo flatpak-builder --run build-dir org.freedesktop.fwupd.json /app/libexec/fwupd/fwupdtool get-devices

# install the single file bundle
# flatpak remote-add --if-not-exists flathub https://dl.flathub.org/repo/flathub.flatpakrepo
# flatpak install fwupd.flatpak

# to run a shell in the same environment that flatpak sees:
# flatpak run --command=sh --devel org.freedesktop.fwupd

# to run fwupdtool as root:
# sudo flatpak run org.freedesktop.fwupd --verbose get-devices



#!/usr/bin/python3
""" Builds a header for the plugins to include """

# pylint: disable=invalid-name,wrong-import-position,pointless-string-statement

"""
SPDX-License-Identifier: LGPL-2.1+
"""

import sys
import hashlib

def usage(return_code):
    """ print usage and exit with the supplied return code """
    if return_code == 0:
        out = sys.stdout
    else:
        out = sys.stderr
    out.write("usage: fu-hash.py <DAEMON> <HEADER>")
    sys.exit(return_code)

if __name__ == '__main__':
    if {'-?', '--help', '--usage'}.intersection(set(sys.argv)):
        usage(0)
    if len(sys.argv) != 3:
        usage(1)
    with open(sys.argv[1], 'rb') as f:
        buf = f.read()
        csum = hashlib.sha256(buf).hexdigest()
        with open(sys.argv[2], 'w') as f2:
            f2.write('#pragma once\n')
            f2.write('#define FU_BUILD_HASH "%s"\n' % csum)
